"""
adjusttable.py

Copyright (C) 2004 by Matsumoto,Tadashi
(E-Mail Address: ma2@city.plala.jp)

Everyone is permitted to do anything on this program
including copying, modifying, improving,
as long as you don't try to pretend that you wrote it.
i.e., the above copyright notice has to appear in all copies.
Binary distribution requires original version messages.
You don't have to ask before copying, redistribution or publishing.
THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE.
"""
from cStringIO import StringIO

jplib_exists = 1
try:
    import jplib
except:
    jplib_exists = 0

if jplib_exists:
    is7bit = jplib.is7bit
    kanjiwidth = jplib.width
else:
    import re
    is7bit = re.compile('^[\000-\177]*$').match
    def kanjiwidth(u):
        w = 0
        for c in u:
           if ord(c) > 255:
               w += 2
           else:
               w += 1
        return w 

def adjustgridtable(lines):
    rlines = []
    for line in lines:
        rline = []
        for col in line.split('|'):
            if is7bit(col):
                rline.append(col)
            else:
                diff = kanjiwidth(col) - len(col)
                if diff:
                    rline.append(col+' '*diff)
        rline = '|'.join(rline)
        rlines.append(rline)
    return rlines

def adjustsimpletable(lines, columns):
    rlines = []
    limit = len(columns)
    for line in lines:
        if is7bit(line):
            rlines.append(line)
            continue
        f = StringIO()
        i = 0
        start, end = columns[0]
        width = 0
        kanji = 0
        for c in line:
            f.write(c.encode('utf-8'))
            if ord(c) > 255:
                width += 2
                kanji += 1
            else:
                width +=1
            if width <= end:
                continue
            else:
                i += 1
                if i < limit:
                    if kanji:
                        f.write(' '*kanji)
                        kanji = 0
                    start, end = columns[i]
                else:
                    end = len(line)*2 + 1
        rline = f.getvalue()
        rlines.append(rline.decode('utf-8'))
    return rlines
